<!DOCTYPE html>
<html lang="en">

<head>
    <meta charset="utf-8">
    <title>
        {{ app_config['web_page']['title']|e }}
    </title>
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <meta http-equiv="X-Clacks-Overhead" content="GNU Terry Pratchett">
    <link rel="preload" href="/static/fonts/noto-sans-display-latin-500.woff2" as="font" type="font/woff2" crossorigin>
    <link rel="preload" href="/static/fonts/fira-code-v12-latin-500.woff2" as="font" type="font/woff2" crossorigin>
    <link rel="preload" href="/static/fonts/fira-code-v12-latin-700.woff2" as="font" type="font/woff2" crossorigin>
    <script type="module" src="/static/health.js"></script>
    <link rel="stylesheet" href="/static/main.css">
    <link rel="icon" type="image/x-icon" href="/favicon.ico">
</head>

<body>
    <main class="contained-content">
        <h1>{{ app_config['web_page']['title']|e }} Health</h1>
        {{ header | safe }}
        <div id="check_box_container"></div>
        <div class="file-list" id="health"></div>
        <div class="file-list" id="profile"></div>
    </main>
</body>

</html>
